#include "SceneLib.h"

#include <windows.h>
#include <stdio.h>
#include "../gl/gl.h"
#include "../gl/glu.h"


void UseOpenGLToFinishPipeline(Scene* scene);
void renderTriangleGL(Triangle *t);
void switchTextureFilteringGL(Scene *s);
void checkErrorsGL(const char *prefix);


void P7_fake(Scene *scene)
{
   UseOpenGLToFinishPipeline(scene);
   return;
}

void UseOpenGLToFinishPipeline(Scene* scene)
{
   TriangleListNode *ptr;

   /* use the Scene object to set OpenGL's viewport */
   glViewport(0, 0, scene->vp_width, scene->vp_height);

   /* set OpenGL's projection matrix */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
//   glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

   /* set OpenGL's modelview matrix */
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glClearColor(0, 100/(float)255, 100/(float)255, 0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);

#if DEBUG1==8
   fprintf(stderr, "================= P7_fake begin ============\n", DEBUG1);
   fflush(stderr);
#endif /* DEBUG1 */

   /* render each Triangle from the list of Triangle objects */
   for (ptr = (scene->head_node).next;  ptr;  ptr = ptr->next)
   {
      renderTriangleGL(ptr->t);
   }

#if DEBUG1==8
   fprintf(stderr, "================= P7_fake end ==============\n", DEBUG1);
   fflush(stderr);
#endif /* DEBUG1 */

   glFinish(); /* wait while everything gets rendered */

   glDisable(GL_DEPTH_TEST);

   checkErrorsGL("OpenGL errors in rendering scene!");

   return;
}


void renderTriangleGL(Triangle *t)
{
   if (t->tex)
   {
      glBindTexture(GL_TEXTURE_2D, t->tex->tex);
      glEnable(GL_TEXTURE_2D);
   }

   /* now draw the triangle */
   glBegin(GL_TRIANGLES);
      if (t->tex) glTexCoord2f(t->v[0].s, t->v[0].t);
      glColor3f ( t->v[0].r, t->v[0].g, t->v[0].b );
      glVertex3f( t->v[0].x, t->v[0].y, t->v[0].z );

      if (t->tex) glTexCoord2f(t->v[1].s, t->v[1].t);
      glColor3f ( t->v[1].r, t->v[1].g, t->v[1].b );
      glVertex3f( t->v[1].x, t->v[1].y, t->v[1].z );

      if (t->tex) glTexCoord2f(t->v[2].s, t->v[2].t);
      glColor3f ( t->v[2].r, t->v[2].g, t->v[2].b );
      glVertex3f( t->v[2].x, t->v[2].y, t->v[2].z );
   glEnd();

#if DEBUG1==8
      fprintf(stderr, "[% .6f % .6f % .6f % .6f]\n", t->v[0].x, t->v[0].y, t->v[0].z, t->v[0].w );
      fprintf(stderr, "{% .6f % .6f % .6f % .6f}\n", t->v[1].x, t->v[1].y, t->v[1].z, t->v[1].w );
      fprintf(stderr, "<% .6f % .6f % .6f % .6f>\n", t->v[2].x, t->v[2].y, t->v[2].z, t->v[2].w );
      fflush(stderr);
#endif /* DEBUG1 */

   if (t->tex)
   {
      glDisable(GL_TEXTURE_2D);
   }

   checkErrorsGL("Errors in rendering triangle!");

   return;
}//renderTriangleGL()


void switchTextureFilteringGL(Scene *s)
{
   /* do this with OpenGL for now */
   int i;
   for (i = 0; i < s->textureCount; i++)
   {
      glBindTexture(GL_TEXTURE_2D, s->textures[i]->tex);
      if (s->bilerp_flag)
      {
         glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
      }
      else
      {
         glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
      }
   }

   return;
}//switchTextureFilteringGL()


void checkErrorsGL(const char *prefix)
{
   GLenum error;
   if ((error = glGetError()) != GL_NO_ERROR)
   {
      fprintf(stderr,"%s: gl error: %s\n",prefix, gluErrorString(error));
      exit(-1);
   }
   return;
}//checkErrorsGL()
